@@ -1,13 +1,14 @@
-import json
 import sys
 import threading
 import time
+from datetime import datetime, timedelta
 from enum import Enum, auto
 from pathlib import Path
-from typing import List, Optional, Dict, Any
+from typing import List, Optional, Dict
 
+import tzlocal
 from apscheduler.events import EVENT_JOB_EXECUTED, EVENT_JOB_ERROR, \
-    EVENT_JOB_ADDED, EVENT_JOB_SUBMITTED, EVENT_JOB_REMOVED
+    EVENT_JOB_ADDED, EVENT_JOB_SUBMITTED, EVENT_JOB_REMOVED, EVENT_JOB_MISSED, EVENT_JOB_MAX_INSTANCES
 from apscheduler.events import JobExecutionEvent, JobEvent, JobSubmissionEvent
 from apscheduler.executors.pool import ThreadPoolExecutor
 from apscheduler.job import Job
@@ -18,8 +19,6 @@ from dotenv import load_dotenv
 from mysql.connector import Error as mysqlError
 from pydantic import BaseModel
 from telegram import Bot
-from datetime import datetime, timedelta
-import tzlocal
 
 from .mysql_basic import *
 
@@ -65,12 +64,12 @@ class JobInfo(BaseModel):
     module: str  # name of the module, where to find the callback function
     args: Optional[List] = []  # args to pass to the function
     kwargs: Optional[Dict] = {}  # kwargs to pass to the function
-    pydantic_args: Optional[List[tuple[str, dict]]] = None  # args tuple of [] will be first parsed by model.parse_obj
+    pydantic_args: Optional[List[tuple[str, Dict]]] = None  # args tuple of [] will be first parsed by model.parse_obj
     pydantic_kwargs: Optional[Dict[str, tuple[str, Dict]]] = None  # kwargs to pass to the function
     trigger: Optional[AllowedTrigger] = AllowedTrigger.date  # the trigger object that controls the schedule of this job
     trigger_config: dict = dict()  # configuration of the triggers
     coalesce: bool = False  # whether to only run the job once when several run times are due
-    misfire_grace_time: Optional[int] = None  # time (in seconds) how much this job’s execution is allowed to be late
+    misfire_grace_time: Optional[int] = 30  # time (in seconds) how much this job’s execution is allowed to be late
     # (None means “allow the job to run no matter how late it is”)
     max_instances: Optional[int] = 1
     run_in_background: Optional[bool] = False
@@ -150,7 +149,7 @@ class Task(BaseModel):
                     target_system=sql_result['target_system'],
                     comment=sql_result['comment'],
                     status=TaskState[sql_result['status']],
-                    job_info=JobInfo(**json.loads(sql_result['job_info'])))
+                    job_info=JobInfo.parse_raw((sql_result['job_info'])))
 
     @classmethod
     def update_task(cls, task_id: int, kwarg):
@@ -167,10 +166,13 @@ class Task(BaseModel):
             logger.error("could not send info from bot")
             logger.error(f"{type(err).__name__}: {err}")
 
-    def execution_info_start(self):
+    def execution_info_start(self, warning: str = None):
         if self.job_info.bot_info is not None:
             if self.status.value >= self.job_info.bot_info.info_level.value:
-                msg = f"{TaskState.executing.symbol} execution started\n"
+                if warning is not None:
+                    msg = f"⚠️ Warning\n<code>{warning}</code>\n\n"
+                else:
+                    msg = f"📍 execution started\n"
                 msg += f"<code>{self.__repr__()}</code>"
                 Task.send_bot_info(self, msg)
 
@@ -236,7 +238,8 @@ class ScheduleThread(threading.Thread):
         }
         self.scheduler = BackgroundScheduler(executors=executors)
         self.scheduler.add_listener(self.listen_job_scheduled, EVENT_JOB_ADDED)
-        self.scheduler.add_listener(self.listen_execution_start, EVENT_JOB_SUBMITTED)
+        self.scheduler.add_listener(self.listen_execution_start,
+                                    EVENT_JOB_SUBMITTED | EVENT_JOB_MISSED | EVENT_JOB_MAX_INSTANCES)
         self.scheduler.add_listener(self.listen_execution_ends, EVENT_JOB_EXECUTED | EVENT_JOB_ERROR)
         self.scheduler.add_listener(self.listen_job_removed, EVENT_JOB_REMOVED)
         if time_zone:
@@ -245,7 +248,6 @@ class ScheduleThread(threading.Thread):
             self.time_zone = str(tzlocal.get_localzone())
         self.tasks: List[Task] = list()
 
-
     def _add_task(self, task: Task):
         self.tasks.append(task)
 
@@ -269,7 +271,14 @@ class ScheduleThread(threading.Thread):
         task = self._get_task_by_id(int(event.job_id))
         task.executions_instances += 1
         # job maybe unavailable from the job store, this means it is remove and had its last run
-        task.execution_info_start()
+        if event.code == EVENT_JOB_MISSED:
+            warning = "job’s execution was missed"
+        elif event.code == EVENT_JOB_MAX_INSTANCES:
+            warning = "job was not accepted by the executor " \
+                      "because its already reached maximum concurrently executing instances"
+        else:
+            warning = None
+        task.execution_info_start(warning=warning)
 
     def listen_execution_ends(self, event: JobExecutionEvent):
         logger.debug("listen_execution_ends")
@@ -367,6 +376,7 @@ class ScheduleThread(threading.Thread):
                                                args=[task],
                                                executor=executor,
                                                max_instances=task.job_info.max_instances,
+                                               misfire_grace_time=task.job_info.misfire_grace_time,
                                                trigger=trigger)
         task.job = scheduler_job
         return scheduler_job
@@ -381,7 +391,8 @@ class ScheduleThread(threading.Thread):
                 task.job_info.args += getattr(sys.modules[task.job_info.module], pyd_arg[0]).parse_obj(pyd_arg[1])
         if task.job_info.pydantic_kwargs is not None:
             for key, pyd_kwarg in task.job_info.pydantic_kwargs.items():
-                task.job_info.kwargs[key] = getattr(sys.modules[task.job_info.module], pyd_kwarg[0]).parse_obj(pyd_kwarg[1])
+                task.job_info.kwargs[key] = getattr(sys.modules[task.job_info.module], pyd_kwarg[0]).parse_obj(
+                    pyd_kwarg[1])
         result = getattr(sys.modules[task.job_info.module], task.job_info.callback)(*task.job_info.args,
                                                                                     **task.job_info.kwargs)
         duration = datetime.now() - tic
